!*****************************************************************************
!                                                                            *
!  BITMAP.F90                                                                *
!                                                                            *
!  This program demonstrates the Fastgraph for Windows 256-color bitmap      *
!  display functions.                                                        *
!                                                                            *
!*****************************************************************************

integer*4 function WinMain (hInstance, hPrevInstance, lpszCmdParam, nCmdShow)
!MS$ATTRIBUTES STDCALL, ALIAS : '_WinMain@16' :: WinMain
use msfwina
use fgwin

interface
integer*4 function WindowProc (window, message, wParam, lParam)
!MS$ATTRIBUTES STDCALL, ALIAS:'_WindowProc@16' :: WindowProc
integer*4 window, message, wParam, lParam
end function WindowProc
end interface

integer*4 hInstance, hPrevInstance, lpszCmdParam, nCmdShow

character*12      AppName
integer*4         iret
integer*4         window
type (T_MSG)      msg
type (T_WNDCLASS) wndclass

data AppName /'FGbitmap'C/

if (hPrevInstance == 0) then
   wndclass%style         = CS_HREDRAW .or. CS_VREDRAW
   wndclass%lpfnWndProc   = LOC(WindowProc)
   wndclass%cbClsExtra    = 0
   wndclass%cbWndExtra    = 0
   wndclass%hInstance     = hInstance
   wndclass%hIcon         = LoadIcon(NULL,IDI_APPLICATION)
   wndclass%hCursor       = LoadCursor(NULL,IDC_ARROW)
   wndclass%hbrBackground = NULL
   wndclass%lpszMenuName  = LOC(AppName)
   wndclass%lpszClassName = LOC(AppName)
   iret = RegisterClass(wndclass)
   lpszCmdParam = lpszCmdParam
end if

window = CreateWindow(      &
   AppName,                 &  ! window class name
   '256-Color Bitmap Demo'C,&  ! window caption
   WS_OVERLAPPEDWINDOW,     &  ! window style
   CW_USEDEFAULT,           &  ! initial x position
   CW_USEDEFAULT,           &  ! initial y position
   CW_USEDEFAULT,           &  ! initial x size
   CW_USEDEFAULT,           &  ! initial y size
   NULL,                    &  ! parent window handle
   NULL,                    &  ! window menu handle
   hInstance,               &  ! program instance handle
   NULL)                       ! creation parameters

iret = ShowWindow(window,nCmdShow)
iret = UpdateWindow(window)

do while (GetMessage(msg,NULL,0,0))
   iret = TranslateMessage(msg)
   iret = DispatchMessage(msg)
end do

WinMain = msg%wParam
return
end

!*****************************************************************************
!                                                                            *
!  WindowProc                                                                *
!                                                                            *
!  Window procedure to handle messages sent to the window.                   *
!                                                                            *
!*****************************************************************************

integer*4 function WindowProc (window, message, wParam, lParam)
!MS$ATTRIBUTES STDCALL, ALIAS:'_WindowProc@16' :: WindowProc
use msfwina
use fgwin
integer*4 window, message, wParam, lParam

include 'bitmap.fi'

type (T_PAINTSTRUCT) ps
integer*4 cxClient, cyClient
integer*4 hdc, hpal, hvb
integer*4 iret

save cxClient, cyClient, hdc, hpal, hvb

WindowProc = 0

select case (message)
   case (WM_CREATE)
      hdc = GetDC(window)
      call fg_setdc(hdc)
      hpal = fg_defpal()
      call fg_realize(hpal)

      call fg_vbinit
      hvb = fg_vballoc(WIDTH,HEIGHT)
      iret = fg_vbopen(hvb)
      call fg_vbcolors

      call fg_setcolor(20)
      call fg_fillpage
      return

   case (WM_COMMAND)
      select case (wParam)
         case (IDM_DRWIMAGE)
            call fg_fillpage()
            call fg_move(WIDTH/2-20,HEIGHT/2+10)
            call fg_drwimage(bird,40,20)
            call fg_vbscale(0,WIDTH-1,0,HEIGHT-1,0,cxClient-1,0,cyClient-1)
            return

         case (IDM_CLPIMAGE)
            call fg_fillpage()
            call fg_move(-20,HEIGHT/2+10)
            call fg_clpimage(bird,40,20)
            call fg_vbscale(0,WIDTH-1,0,HEIGHT-1,0,cxClient-1,0,cyClient-1)
            return

         case (IDM_REVIMAGE)
            call fg_fillpage()
            call fg_move(WIDTH/2-20,HEIGHT/2+10)
            call fg_revimage(bird,40,20)
            call fg_vbscale(0,WIDTH-1,0,HEIGHT-1,0,cxClient-1,0,cyClient-1)
            return

         case (IDM_FLPIMAGE)
            call fg_fillpage()
            call fg_move(WIDTH-20,HEIGHT/2+10)
            call fg_flpimage(bird,40,20)
            call fg_vbscale(0,WIDTH-1,0,HEIGHT-1,0,cxClient-1,0,cyClient-1)
            return

         case (IDM_PUTIMAGE)
            call fg_fillpage()
            call fg_move(WIDTH/2-20,HEIGHT/2+10)
            call fg_putimage(bird,40,20)
            call fg_vbscale(0,WIDTH-1,0,HEIGHT-1,0,cxClient-1,0,cyClient-1)
            return

         case (IDM_QUIT)
            iret = SendMessage(window,WM_CLOSE,0,0)
            return
      end select

   case (WM_PAINT)
      iret = BeginPaint(window,ps)
      call fg_vbscale(0,WIDTH-1,0,HEIGHT-1,0,cxClient-1,0,cyClient-1)
      iret = EndPaint(window,ps)
      return

   case (WM_SETFOCUS)
      call fg_realize(hpal)
      iret = InvalidateRect(window,NULL_RECT,.TRUE.)
      return

   case (WM_SIZE)
      cxClient = LOWORD(lParam)
      cyClient = HIWORD(lParam)
      return

   case (WM_DESTROY)
      call fg_vbclose
      call fg_vbfree(hvb)
      call fg_vbfin
      iret = DeleteObject(hpal)
      iret = ReleaseDC(window,hdc)
      call PostQuitMessage(0)
      return
end select

WindowProc = DefWindowProc(window,message,wParam,lParam)
return
end

!*****************************************************************************

block data

include 'bitmap.fi'

! 40x20 256-color bitmapped image of a bird
data bird / &
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, &
    0, 0,10,10, 0,10, 0,10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, &
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, &
    0, 0,10,25,10,17,10,17,10,10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, &
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, &
    0,10,25,25,10,17,10,17,10,25,10, 0, 0, 0, 0, 0, 0, 0, 0, 0, &
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, &
   10,25,25,25,25,10,25,10,25,25,10, 0, 0, 0, 0, 0, 0, 0, 0, 0, &
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,10,10,10, &
   10,25,25,25,25,10,25,10,25,10, 0, 0, 0, 0, 0, 0, 0,10,10, 0, &
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,10,25,25,25, &
   10,25,25,25,25,25,25,25,25,10, 0, 0,10,10,10,10,10,17,10, 0, &
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,10,10,25,25,10, &
   25,25,25,25,25,25,25,25,10, 0, 0,10,25,25,25,25,25,10, 0, 0, &
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,10,25,25,10,25,25, &
   10,25,25,25,25,25,25,25,10, 0,10,25,25,25,25,25,10, 0, 0, 0, &
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,10,25,10,25,25,10,25, &
   25,25,25,25,25,25,25,25,10, 0,10,25,25,25,25,10, 0, 0, 0, 0, &
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,10,25,25,10,25,25,10, &
   25,25,25,25,25,25,25,25,10,10,25,25,25,25,10, 0, 0, 0, 0, 0, &
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,10,25,10,25,25,10,25, &
   25,25,25,25,25,25,25,10,25,25,25,25,25,10, 0, 0, 0, 0, 0, 0, &
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,10,10,25,25,10,25,25,25,25, &
   25,25,25,25,25,25,25,10,25,25,10,25,10, 0, 0, 0, 0, 0, 0, 0, &
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,10,25,10,25,25,25,25,25,25,25, &
   25,25,25,25,25,25,25,10,25,25,25,25,10, 0, 0, 0, 0, 0, 0, 0, &
    0,10,10,10,10, 0, 0, 0, 0, 0,10,25,25,10,25,25,25,25,25,25, &
   25,25,25,25,25,25,25,25,25,10,25,10, 0, 0, 0, 0, 0, 0, 0, 0, &
   10,17,17,17,17,10,10,10,10, 0,10,10,25,25,25,25,25,25,25,25, &
   25,25,25,25,25,10,25,25,25,10,10,10,10, 0, 0, 0, 0, 0, 0, 0, &
    0,10,17,17,17,17,17,17,17,10,25,25,10,25,25,25,25,25,25,25, &
   25,25,25,25,10,25,25,25,10,17,17,17,17,10,10,10,10,10,10, 0, &
    0, 0,10,10,10,10,25,25,25,25,25,25,25,25,25,25,25,25,25,25, &
   25,25,25,25,25,25,25,25,25,25,25,25,25,25,10,14,14,10, 0, 0, &
    0, 0, 0, 0, 0, 0,10,10,10,10,10,10,10,10,25,25,25,25,25,25, &
   25,25,25,25,25,25,25,25,25,25,25,25,14,25,25,10,10, 0, 0, 0, &
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,10,10,25,25,25,25, &
   25,25,25,25,25,25,25,10,10,10,10,25,25,25,10, 0, 0, 0, 0, 0, &
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,10,10,10,10, &
   10,10,10,10,10,10,10, 0, 0, 0, 0,10,10,10, 0, 0, 0, 0, 0, 0/

end
