!*****************************************************************************
!                                                                            *
!  BITMAP16.F90                                                              *
!                                                                            *
!  This program demonstrates how to translate a 16-color bitmap to the 256-  *
!  color format, and display the resulting 256-color image.                  *
!                                                                            *
!*****************************************************************************

integer*4 function WinMain (hInstance, hPrevInstance, lpszCmdParam, nCmdShow)
!MS$ATTRIBUTES STDCALL, ALIAS : '_WinMain@16' :: WinMain
use msfwina
use fgwin

interface
integer*4 function WindowProc (window, message, wParam, lParam)
!MS$ATTRIBUTES STDCALL, ALIAS:'_WindowProc@16' :: WindowProc
integer*4 window, message, wParam, lParam
end function WindowProc
end interface

integer*4 hInstance, hPrevInstance, lpszCmdParam, nCmdShow

character*12      AppName
integer*4         iret
integer*4         window
type (T_MSG)      msg
type (T_WNDCLASS) wndclass

data AppName /'FGbitmap16'C/

if (hPrevInstance == 0) then
   wndclass%style         = CS_HREDRAW .or. CS_VREDRAW
   wndclass%lpfnWndProc   = LOC(WindowProc)
   wndclass%cbClsExtra    = 0
   wndclass%cbWndExtra    = 0
   wndclass%hInstance     = hInstance
   wndclass%hIcon         = LoadIcon(NULL,IDI_APPLICATION)
   wndclass%hCursor       = LoadCursor(NULL,IDC_ARROW)
   wndclass%hbrBackground = NULL
   wndclass%lpszMenuName  = NULL
   wndclass%lpszClassName = LOC(AppName)
   iret = RegisterClass(wndclass)
   lpszCmdParam = lpszCmdParam
end if

window = CreateWindow(      &
   AppName,                 &  ! window class name
   '16-Color Bitmap Demo'C, &  ! window caption
   WS_OVERLAPPEDWINDOW,     &  ! window style
   CW_USEDEFAULT,           &  ! initial x position
   CW_USEDEFAULT,           &  ! initial y position
   CW_USEDEFAULT,           &  ! initial x size
   CW_USEDEFAULT,           &  ! initial y size
   NULL,                    &  ! parent window handle
   NULL,                    &  ! window menu handle
   hInstance,               &  ! program instance handle
   NULL)                       ! creation parameters

iret = ShowWindow(window,nCmdShow)
iret = UpdateWindow(window)

do while (GetMessage(msg,NULL,0,0))
   iret = TranslateMessage(msg)
   iret = DispatchMessage(msg)
end do

WinMain = msg%wParam
return
end

!*****************************************************************************
!                                                                            *
!  WindowProc                                                                *
!                                                                            *
!  Window procedure to handle messages sent to the window.                   *
!                                                                            *
!*****************************************************************************

integer*4 function WindowProc (window, message, wParam, lParam)
!MS$ATTRIBUTES STDCALL, ALIAS:'_WindowProc@16' :: WindowProc
use msfwina
use fgwin
integer*4 window, message, wParam, lParam

include 'bitmap16.fi'

type (T_PAINTSTRUCT) ps
integer*4 cxClient, cyClient
integer*4 hdc, hpal, hvb
integer*4 i
integer*4 iret

save cxClient, cyClient, hdc, hpal, hvb

WindowProc = 0

select case (message)
   case (WM_CREATE)
      hdc = GetDC(window)
      call fg_setdc(hdc)
      hpal = fg_defpal()
      call fg_realize(hpal)

      call fg_vbinit
      hvb = fg_vballoc(WIDTH,HEIGHT)
      iret = fg_vbopen(hvb)
      call fg_vbcolors

      call fg_unpack(bird16,bird256,20*20)
      do i = 1,40*20
         bird256(i) = translation(bird256(i))
      end do
         
      call fg_setcolor(20)
      call fg_fillpage
      call fg_move(WIDTH/2-20,HEIGHT/2+10)
      call fg_drwimage(bird256,40,20)

      return

   case (WM_PAINT)
      iret = BeginPaint(window,ps)
      call fg_vbscale(0,WIDTH-1,0,HEIGHT-1,0,cxClient-1,0,cyClient-1)
      iret = EndPaint(window,ps)
      return

   case (WM_SETFOCUS)
      call fg_realize(hpal)
      iret = InvalidateRect(window,NULL_RECT,.TRUE.)
      return

   case (WM_SIZE)
      cxClient = LOWORD(lParam)
      cyClient = HIWORD(lParam)
      return

   case (WM_DESTROY)
      call fg_vbclose
      call fg_vbfree(hvb)
      call fg_vbfin
      iret = DeleteObject(hpal)
      iret = ReleaseDC(window,hdc)
      call PostQuitMessage(0)
      return
end select

WindowProc = DefWindowProc(window,message,wParam,lParam)
return
end

!*****************************************************************************

block data

include 'bitmap16.fi'

! 40x20 pixel (20x20 byte) 16-color bitmapped image of a bird
data bird16 / &
   #00,#00,#00,#00,#00,#00,#00,#00,#00,#00, &
   #00,#11,#01,#01,#00,#00,#00,#00,#00,#00, &
   #00,#00,#00,#00,#00,#00,#00,#00,#00,#00, &
   #00,#1F,#17,#17,#11,#00,#00,#00,#00,#00, &
   #00,#00,#00,#00,#00,#00,#00,#00,#00,#00, &
   #01,#FF,#17,#17,#1F,#10,#00,#00,#00,#00, &
   #00,#00,#00,#00,#00,#00,#00,#00,#00,#00, &
   #1F,#FF,#F1,#F1,#FF,#10,#00,#00,#00,#00, &
   #00,#00,#00,#00,#00,#00,#00,#00,#01,#11, &
   #1F,#FF,#F1,#F1,#F1,#00,#00,#00,#01,#10, &
   #00,#00,#00,#00,#00,#00,#00,#00,#1F,#FF, &
   #1F,#FF,#FF,#FF,#F1,#00,#11,#11,#17,#10, &
   #00,#00,#00,#00,#00,#00,#00,#01,#1F,#F1, &
   #FF,#FF,#FF,#FF,#10,#01,#FF,#FF,#F1,#00, &
   #00,#00,#00,#00,#00,#00,#00,#1F,#F1,#FF, &
   #1F,#FF,#FF,#FF,#10,#1F,#FF,#FF,#10,#00, &
   #00,#00,#00,#00,#00,#00,#01,#F1,#FF,#1F, &
   #FF,#FF,#FF,#FF,#10,#1F,#FF,#F1,#00,#00, &
   #00,#00,#00,#00,#00,#00,#01,#FF,#1F,#F1, &
   #FF,#FF,#FF,#FF,#11,#FF,#FF,#10,#00,#00, &
   #00,#00,#00,#00,#00,#00,#01,#F1,#FF,#1F, &
   #FF,#FF,#FF,#F1,#FF,#FF,#F1,#00,#00,#00, &
   #00,#00,#00,#00,#00,#01,#1F,#F1,#FF,#FF, &
   #FF,#FF,#FF,#F1,#FF,#1F,#10,#00,#00,#00, &
   #00,#00,#00,#00,#00,#1F,#1F,#FF,#FF,#FF, &
   #FF,#FF,#FF,#F1,#FF,#FF,#10,#00,#00,#00, &
   #01,#11,#10,#00,#00,#1F,#F1,#FF,#FF,#FF, &
   #FF,#FF,#FF,#FF,#F1,#F1,#00,#00,#00,#00, &
   #17,#77,#71,#11,#10,#11,#FF,#FF,#FF,#FF, &
   #FF,#FF,#F1,#FF,#F1,#11,#10,#00,#00,#00, &
   #01,#77,#77,#77,#71,#FF,#1F,#FF,#FF,#FF, &
   #FF,#FF,#1F,#FF,#17,#77,#71,#11,#11,#10, &
   #00,#11,#11,#FF,#FF,#FF,#FF,#FF,#FF,#FF, &
   #FF,#FF,#FF,#FF,#FF,#FF,#FF,#14,#41,#00, &
   #00,#00,#00,#11,#11,#11,#11,#FF,#FF,#FF, &
   #FF,#FF,#FF,#FF,#FF,#FF,#4F,#F1,#10,#00, &
   #00,#00,#00,#00,#00,#00,#00,#11,#FF,#FF, &
   #FF,#FF,#FF,#F1,#11,#1F,#FF,#10,#00,#00, &
   #00,#00,#00,#00,#00,#00,#00,#00,#11,#11, &
   #11,#11,#11,#10,#00,#01,#11,#00,#00,#00/

! bitmap translation table
data translation /0,10,12,13,14,15,16,17,18,19,20,21,22,23,24,25/

end
