!*****************************************************************************
!                                                                            *
!  FADE.F90                                                                  *
!                                                                            *
!  This program shows how perform a palette fade.                            *
!                                                                            *
!*****************************************************************************

integer*4 function WinMain (hInstance, hPrevInstance, lpszCmdParam, nCmdShow)
!MS$ATTRIBUTES STDCALL, ALIAS : '_WinMain@16' :: WinMain
use msfwina
use fgwin

interface
integer*4 function WindowProc (window, message, wParam, lParam)
!MS$ATTRIBUTES STDCALL, ALIAS:'_WindowProc@16' :: WindowProc
integer*4 window, message, wParam, lParam
end function WindowProc
end interface

integer*4 hInstance, hPrevInstance, lpszCmdParam, nCmdShow

character*12      AppName
integer*4         iret
integer*4         window
type (T_MSG)      msg
type (T_WNDCLASS) wndclass

data AppName /'FGfade'C/

if (hPrevInstance == 0) then
   wndclass%style         = CS_HREDRAW .or. CS_VREDRAW
   wndclass%lpfnWndProc   = LOC(WindowProc)
   wndclass%cbClsExtra    = 0
   wndclass%cbWndExtra    = 0
   wndclass%hInstance     = hInstance
   wndclass%hIcon         = LoadIcon(NULL,IDI_APPLICATION)
   wndclass%hCursor       = LoadCursor(NULL,IDC_ARROW)
   wndclass%hbrBackground = NULL
   wndclass%lpszMenuName  = LOC(AppName)
   wndclass%lpszClassName = LOC(AppName)
   iret = RegisterClass(wndclass)
   lpszCmdParam = lpszCmdParam
end if

window = CreateWindow(      &
   AppName,                 &  ! window class name
   'Palette Fades'C,        &  ! window caption
   WS_OVERLAPPEDWINDOW,     &  ! window style
   CW_USEDEFAULT,           &  ! initial x position
   CW_USEDEFAULT,           &  ! initial y position
   CW_USEDEFAULT,           &  ! initial x size
   CW_USEDEFAULT,           &  ! initial y size
   NULL,                    &  ! parent window handle
   NULL,                    &  ! window menu handle
   hInstance,               &  ! program instance handle
   NULL)                       ! creation parameters

iret = ShowWindow(window,nCmdShow)
iret = UpdateWindow(window)

do while (GetMessage(msg,NULL,0,0))
   iret = TranslateMessage(msg)
   iret = DispatchMessage(msg)
end do

WinMain = msg%wParam
return
end

!*****************************************************************************
!                                                                            *
!  WindowProc                                                                *
!                                                                            *
!  Window procedure to handle messages sent to the window.                   *
!                                                                            *
!*****************************************************************************

integer*4 function WindowProc (window, message, wParam, lParam)
!MS$ATTRIBUTES STDCALL, ALIAS:'_WindowProc@16' :: WindowProc
use msfwina
use fgwin
integer*4 window, message, wParam, lParam

include 'fade.fi'

type (T_PAINTSTRUCT) ps
integer*4 hdc, hpal, hvb
integer*4 iret

save hdc, hpal, hvb

WindowProc = 0

select case (message)
   case (WM_CREATE)
      hdc = GetDC(window)
      call fg_setdc(hdc)
      hpal = fg_defpal()
      call fg_realize(hpal)

      call fg_vbinit
      hvb = fg_vballoc(WIDTH,HEIGHT)
      iret = fg_vbopen(hvb)
      call fg_vbcolors

      iret = fg_showpcx('MOUSE.PCX'C,2)
      call fg_getdacs(10,236,original)

      hmenu = GetMenu(window)
      return

   case (WM_COMMAND)
      select case (wParam)
         case (IDM_FADEIN)
            call do_fadein
            return

         case (IDM_FADEOUT)
            call do_fadeout
            return

         case (IDM_QUIT)
            iret = SendMessage(window,WM_CLOSE,0,0)
            return
      end select

   case (WM_PAINT)
      iret = BeginPaint(window,ps)
      call fg_vbscale(0,WIDTH-1,0,HEIGHT-1,0,cxClient-1,0,cyClient-1)
      iret = EndPaint(window,ps)
      return

   case (WM_SETFOCUS)
      call fg_realize(hpal)
      iret = InvalidateRect(window,NULL_RECT,.TRUE.)
      return

   case (WM_SIZE)
      cxClient = LOWORD(lParam)
      cyClient = HIWORD(lParam)
      return

   case (WM_DESTROY)
      call fg_vbclose
      call fg_vbfree(hvb)
      call fg_vbfin
      iret = DeleteObject(hpal)
      iret = ReleaseDC(window,hdc)
      call PostQuitMessage(0)
      return
end select

WindowProc = DefWindowProc(window,message,wParam,lParam)
return
end

!*****************************************************************************
!                                                                            *
!  do_fadein                                                                 *
!                                                                            *
!  Fade an image back to its original colors.                                *
!                                                                            *
!*****************************************************************************

subroutine do_fadein
use msfwina
use fgwin

include 'fade.fi'

integer*4 i
logical*1 fading

do i = 0,236*3-1
   current(i) = 0
end do

fading = .TRUE.
do while (fading)
   fading = .FALSE.
   do i = 0,236*3-1
      if (current(i) /= original(i)) then
         current(i) = current(i) + 1
         fading = .TRUE.
      end if
   end do
   call fg_setdacs(10,236,current)
   if (fg_colors() > 8) then
      call fg_vbscale(0,WIDTH-1,0,HEIGHT-1,0,cxClient-1,0,cyClient-1)
   end if
end do

i = EnableMenuItem(hmenu,IDM_FADEIN,MF_GRAYED)
i = EnableMenuItem(hmenu,IDM_FADEOUT,MF_ENABLED)
i = DrawMenuBar(GetActiveWindow())

return
end

!*****************************************************************************
!                                                                            *
!  do_fadeout                                                                *
!                                                                            *
!  Fade an image to black.                                                   *
!                                                                            *
!*****************************************************************************

subroutine do_fadeout
use msfwina
use fgwin

include 'fade.fi'

integer*4 i
logical*1 fading

do i = 0,236*3-1
   current(i) = original(i)
end do

fading = .TRUE.
do while (fading)
   fading = .FALSE.
   do i = 0,236*3-1
      if (current(i) /= 0) then
         current(i) = current(i) - 1
         fading = .TRUE.
      end if
   end do
   call fg_setdacs(10,236,current)
   if (fg_colors() > 8) then
      call fg_vbscale(0,WIDTH-1,0,HEIGHT-1,0,cxClient-1,0,cyClient-1)
   end if
end do

i = EnableMenuItem(hmenu,IDM_FADEOUT,MF_GRAYED)
i = EnableMenuItem(hmenu,IDM_FADEIN,MF_ENABLED)
i = DrawMenuBar(GetActiveWindow())

return
end
