!*****************************************************************************
!                                                                            *
!  FONTDEMO.F90                                                              *
!                                                                            *
!  This program shows how use Windows stock fonts in Fastgraph for Windows,  *
!  and also how to create and use a Windows logical font. The logical font   *
!  is a 24x12 script font.                                                   *
!                                                                            *
!*****************************************************************************

integer*4 function WinMain (hInstance, hPrevInstance, lpszCmdParam, nCmdShow)
!MS$ATTRIBUTES STDCALL, ALIAS : '_WinMain@16' :: WinMain
use msfwina
use fgwin

interface
integer*4 function WindowProc (window, message, wParam, lParam)
!MS$ATTRIBUTES STDCALL, ALIAS:'_WindowProc@16' :: WindowProc
integer*4 window, message, wParam, lParam
end function WindowProc
end interface

integer*4 hInstance, hPrevInstance, lpszCmdParam, nCmdShow

include 'fontdemo.fi'

character*12      AppName
integer*4         hfont
integer*4         iret
integer*4         window
type (T_MSG)      msg
type (T_WNDCLASS) wndclass
type (T_LOGFONT)  lf
common hfont

character*8 typeface
data typeface /'script'C/

data AppName /'FGfontdemo'C/

if (hPrevInstance == 0) then
   wndclass%style         = CS_HREDRAW .or. CS_VREDRAW
   wndclass%lpfnWndProc   = LOC(WindowProc)
   wndclass%cbClsExtra    = 0
   wndclass%cbWndExtra    = 0
   wndclass%hInstance     = hInstance
   wndclass%hIcon         = LoadIcon(NULL,IDI_APPLICATION)
   wndclass%hCursor       = LoadCursor(NULL,IDC_ARROW)
   wndclass%hbrBackground = NULL
   wndclass%lpszMenuName  = NULL
   wndclass%lpszClassName = LOC(AppName)
   iret = RegisterClass(wndclass)
   lpszCmdParam = lpszCmdParam
end if

lf%lfHeight = 24                 ! character height in pixels
lf%lfWidth = 12                  ! character width in pixels
lf%lfEscapement = 0              ! orientation of next character
lf%lfOrientation = 0             ! orientation of first character
lf%lfWeight = FW_NORMAL          ! normal thickness
lf%lfItalic = .FALSE.            ! not italic characters
lf%lfUnderline = .FALSE.         ! not underlined characters
lf%lfStrikeOut = .FALSE.         ! not strikeout characters
lf%lfCharSet = OEM_CHARSET       ! OEM character set
lf%lfOutPrecision = 0            ! default output precision
lf%lfClipPrecision = 0           ! default clipping precision
lf%lfQuality = DEFAULT_QUALITY   ! default scaling quality
lf%lfPitchAndFamily = FF_SCRIPT  ! default pitch, script font family
lf%lfFaceName = LOC(typeface)    ! script typeface
hfont = CreateFontIndirect(lf)

window = CreateWindow(      &
   AppName,                 &  ! window class name
   'Fastgraph for Windows Font Demo'C, &  ! window caption
   WS_OVERLAPPEDWINDOW,     &  ! window style
   CW_USEDEFAULT,           &  ! initial x position
   CW_USEDEFAULT,           &  ! initial y position
   WIDTH,                   &  ! initial x size
   HEIGHT,                  &  ! initial y size
   NULL,                    &  ! parent window handle
   NULL,                    &  ! window menu handle
   hInstance,               &  ! program instance handle
   NULL)                       ! creation parameters

iret = ShowWindow(window,nCmdShow)
iret = UpdateWindow(window)

do while (GetMessage(msg,NULL,0,0))
   iret = TranslateMessage(msg)
   iret = DispatchMessage(msg)
end do

WinMain = msg%wParam
return
end

!*****************************************************************************
!                                                                            *
!  WindowProc                                                                *
!                                                                            *
!  Window procedure to handle messages sent to the window.                   *
!                                                                            *
!*****************************************************************************

integer*4 function WindowProc (window, message, wParam, lParam)
!MS$ATTRIBUTES STDCALL, ALIAS:'_WindowProc@16' :: WindowProc
use msfwina
use fgwin
integer*4 window, message, wParam, lParam

include 'fontdemo.fi'

type (T_PAINTSTRUCT) ps
integer*4 cxClient, cyClient
integer*4 hdc, hfont, hpal, hvb
integer*4 iret
common hfont

save cxClient, cyClient, hdc, hpal, hvb

WindowProc = 0

select case (message)
   case (WM_CREATE)
      hdc = GetDC(window)
      call fg_setdc(hdc)
      hpal = fg_defpal()
      call fg_realize(hpal)

      call fg_vbinit
      hvb = fg_vballoc(WIDTH,HEIGHT)
      iret = fg_vbopen(hvb)
      call fg_vbcolors

      call fg_setcolor(24)
      call fg_fillpage
      return

   case (WM_PAINT)
      iret = BeginPaint(window,ps)
      call fg_vbscale(0,fg_getmaxx(),0,fg_getmaxy(),0,cxClient-1,0,cyClient-1)
      call fg_setcolor(19)
      x = fg_xclient(20)

      call fg_move(x,fg_yclient(20))
      call fg_fontload(10)
      call fg_print('OEM fixed font'C,14)

      call fg_move(x,fg_yclient(40))
      call fg_fontload(11)
      call fg_print('ANSI fixed font'C,15)

      call fg_move(x,fg_yclient(60))
      call fg_fontload(12)
      call fg_print('ANSI var font'C,13)

      call fg_move(x,fg_yclient(80))
      call fg_fontload(13)
      call fg_print('system font'C,11)

      call fg_move(x,fg_yclient(100))
      call fg_fontload(14)
      call fg_print('device default font'C,19)

      call fg_move(x,fg_yclient(120))
      call fg_fontload(16)
      call fg_print('system fixed font'C,17)

      call fg_move(x,fg_yclient(160))
      call fg_logfont(hfont)
      call fg_print('script font'C,11)

      iret = EndPaint(window,ps)
      return

   case (WM_SETFOCUS)
      call fg_realize(hpal)
      iret = InvalidateRect(window,NULL_RECT,.TRUE.)
      return

   case (WM_SIZE)
      cxClient = LOWORD(lParam)
      cyClient = HIWORD(lParam)
      return

   case (WM_DESTROY)
      call fg_vbclose
      call fg_vbfree(hvb)
      call fg_vbfin
      iret = DeleteObject(hfont)
      iret = DeleteObject(hpal)
      iret = ReleaseDC(window,hdc)
      call PostQuitMessage(0)
      return
end select

WindowProc = DefWindowProc(window,message,wParam,lParam)
return
end
