!*****************************************************************************
!                                                                            *
!  GDIDEMO.F90                                                               *
!                                                                            *
!  This program shows how to use Windows GDI functions to write to a virtual *
!  buffer. It uses GDI functions to display a cross-hatched rectangle with a *
!  border, something that would require construction from several graphics   *
!  primitives if using Fastgraph for Windows drawing functions only.         *
!                                                                            *
!  Only the Fastgraph for Windows WinG support libraries allow writing to a  *
!  virtual buffer with GDI functions.                                        *
!                                                                            *
!*****************************************************************************

integer*4 function WinMain (hInstance, hPrevInstance, lpszCmdParam, nCmdShow)
!MS$ATTRIBUTES STDCALL, ALIAS : '_WinMain@16' :: WinMain
use msfwina
use fgwin

interface
integer*4 function WindowProc (window, message, wParam, lParam)
!MS$ATTRIBUTES STDCALL, ALIAS:'_WindowProc@16' :: WindowProc
integer*4 window, message, wParam, lParam
end function WindowProc
end interface

integer*4 hInstance, hPrevInstance, lpszCmdParam, nCmdShow

character*12      AppName
integer*4         hBrush, hDIB, hPen
integer*4         iret
integer*4         window
type (T_MSG)      msg
type (T_WNDCLASS) wndclass

data AppName /'FGgdidemo'C/

if (hPrevInstance == 0) then
   wndclass%style         = CS_HREDRAW .or. CS_VREDRAW
   wndclass%lpfnWndProc   = LOC(WindowProc)
   wndclass%cbClsExtra    = 0
   wndclass%cbWndExtra    = 0
   wndclass%hInstance     = hInstance
   wndclass%hIcon         = LoadIcon(NULL,IDI_APPLICATION)
   wndclass%hCursor       = LoadCursor(NULL,IDC_ARROW)
   wndclass%hbrBackground = NULL
   wndclass%lpszMenuName  = NULL
   wndclass%lpszClassName = LOC(AppName)
   iret = RegisterClass(wndclass)
   lpszCmdParam = lpszCmdParam
end if

window = CreateWindow(      &
   AppName,                 &  ! window class name
   'Fastgraph for Windows GDI Demo'C, &  ! window caption
   WS_OVERLAPPEDWINDOW,     &  ! window style
   CW_USEDEFAULT,           &  ! initial x position
   CW_USEDEFAULT,           &  ! initial y position
   CW_USEDEFAULT,           &  ! initial x size
   CW_USEDEFAULT,           &  ! initial y size
   NULL,                    &  ! parent window handle
   NULL,                    &  ! window menu handle
   hInstance,               &  ! program instance handle
   NULL)                       ! creation parameters

! use FG to fill the virtual buffer with light green pixels
call fg_setcolor(20)
call fg_fillpage

! fg_getdc() returns the WinG device context to which the GDI
! functions will draw, or zero if WinG is not available
hDIB = fg_getdc()
if (hDIB == 0) then
   iret = MessageBox(window,'WinG not available!'C,'GDIDEMO'C,MB_ICONSTOP.or.MB_OK)
   iret = DestroyWindow(window)
   WinMain = 0
   return
end if

! use Windows GDI functions to display a cross-hatched
! rectangle with blue border in the virtual buffer
hPen = CreatePen(PS_SOLID,3,RGB(0,0,255))
hBrush = CreateHatchBrush(HS_DIAGCROSS,RGB(255,0,0))
iret = SelectObject(hDIB,hPen)
iret = SelectObject(hDIB,hBrush)
iret = Rectangle(hDIB,20,20,50,50)
iret = DeleteObject(hPen)
iret = DeleteObject(hBrush)

iret = ShowWindow(window,nCmdShow)
iret = UpdateWindow(window)

do while (GetMessage(msg,NULL,0,0))
   iret = TranslateMessage(msg)
   iret = DispatchMessage(msg)
end do

WinMain = msg%wParam
return
end

!*****************************************************************************
!                                                                            *
!  WindowProc                                                                *
!                                                                            *
!  Window procedure to handle messages sent to the window.                   *
!                                                                            *
!*****************************************************************************

integer*4 function WindowProc (window, message, wParam, lParam)
!MS$ATTRIBUTES STDCALL, ALIAS:'_WindowProc@16' :: WindowProc
use msfwina
use fgwin
integer*4 window, message, wParam, lParam

type (T_PAINTSTRUCT) ps
integer*4 cxClient, cyClient
integer*4 hdc, hpal, hvb
integer*4 iret

save cxClient, cyClient, hdc, hpal, hvb

WindowProc = 0

select case (message)
   case (WM_CREATE)
      hdc = GetDC(window)
      call fg_setdc(hdc)
      hpal = fg_defpal()
      call fg_realize(hpal)

      call fg_vbinit
      hvb = fg_vballoc(320,200)
      iret = fg_vbopen(hvb)
      call fg_vbcolors
      return

   case (WM_PAINT)
      iret = BeginPaint(window,ps)
      call fg_vbscale(0,fg_getmaxx(),0,fg_getmaxy(),0,cxClient-1,0,cyClient-1)
      iret = EndPaint(window,ps)
      return

   case (WM_SETFOCUS)
      call fg_realize(hpal)
      iret = InvalidateRect(window,NULL_RECT,.TRUE.)
      return

   case (WM_SIZE)
      cxClient = LOWORD(lParam)
      cyClient = HIWORD(lParam)
      return

   case (WM_DESTROY)
      call fg_vbclose
      call fg_vbfree(hvb)
      call fg_vbfin
      iret = DeleteObject(hpal)
      iret = ReleaseDC(window,hdc)
      call PostQuitMessage(0)
      return
end select

WindowProc = DefWindowProc(window,message,wParam,lParam)
return
end
