!*****************************************************************************
!                                                                            *
!  GRAPHICS.F90                                                              *
!                                                                            *
!  This program demonstrates some of the Fastgraph for Windows graphics      *
!  primitive functions.                                                      *
!  FADE.F90                                                                  *
!                                                                            *
!  This program shows how perform a palette fade.                            *
!                                                                            *
!*****************************************************************************

integer*4 function WinMain (hInstance, hPrevInstance, lpszCmdParam, nCmdShow)
!MS$ATTRIBUTES STDCALL, ALIAS : '_WinMain@16' :: WinMain
use msfwina
use fgwin

interface
integer*4 function WindowProc (window, message, wParam, lParam)
!MS$ATTRIBUTES STDCALL, ALIAS:'_WindowProc@16' :: WindowProc
integer*4 window, message, wParam, lParam
end function WindowProc
end interface

integer*4 hInstance, hPrevInstance, lpszCmdParam, nCmdShow

include 'graphics.fi'

character*12      AppName
integer*4         iret
integer*4         window
type (T_MSG)      msg
type (T_WNDCLASS) wndclass

data AppName /'FGgraphics'C/

if (hPrevInstance == 0) then
   wndclass%style         = CS_HREDRAW .or. CS_VREDRAW
   wndclass%lpfnWndProc   = LOC(WindowProc)
   wndclass%cbClsExtra    = 0
   wndclass%cbWndExtra    = 0
   wndclass%hInstance     = hInstance
   wndclass%hIcon         = LoadIcon(NULL,IDI_APPLICATION)
   wndclass%hCursor       = LoadCursor(NULL,IDC_ARROW)
   wndclass%hbrBackground = NULL
   wndclass%lpszMenuName  = LOC(AppName)
   wndclass%lpszClassName = LOC(AppName)
   iret = RegisterClass(wndclass)
   lpszCmdParam = lpszCmdParam
end if

window = CreateWindow(      &
   AppName,                 &  ! window class name
   'Graphics Primitives'C,  &  ! window caption
   WS_OVERLAPPEDWINDOW,     &  ! window style
   CW_USEDEFAULT,           &  ! initial x position
   CW_USEDEFAULT,           &  ! initial y position
   WIDTH,                   &  ! initial x size
   HEIGHT,                  &  ! initial y size
   NULL,                    &  ! parent window handle
   NULL,                    &  ! window menu handle
   hInstance,               &  ! program instance handle
   NULL)                       ! creation parameters

iret = ShowWindow(window,nCmdShow)
iret = UpdateWindow(window)

do while (GetMessage(msg,NULL,0,0))
   iret = TranslateMessage(msg)
   iret = DispatchMessage(msg)
end do

WinMain = msg%wParam
return
end

!*****************************************************************************
!                                                                            *
!  WindowProc                                                                *
!                                                                            *
!  Window procedure to handle messages sent to the window.                   *
!                                                                            *
!*****************************************************************************

integer*4 function WindowProc (window, message, wParam, lParam)
!MS$ATTRIBUTES STDCALL, ALIAS:'_WindowProc@16' :: WindowProc
use msfwina
use fgwin
integer*4 window, message, wParam, lParam

include 'graphics.fi'

type (T_PAINTSTRUCT) ps
integer*4 hdc, hpal, hvb
integer*4 iret

save hdc, hpal, hvb

WindowProc = 0

select case (message)
   case (WM_CREATE)
      hdc = GetDC(window)
      call fg_setdc(hdc)
      hpal = fg_defpal()
      call fg_realize(hpal)

      call fg_vbinit
      hvb = fg_vballoc(WIDTH,HEIGHT)
      iret = fg_vbopen(hvb)
      call fg_vbcolors

      call fg_setcolor(25)
      call fg_fillpage
      return

   case (WM_COMMAND)
      select case (wParam)
         case (IDM_POINTS)
            call do_points
            return

         case (IDM_LINES)
            call do_lines
            return

         case (IDM_RECTANGLES)
            call do_rectangles
            return

         case (IDM_CIRCLES)
            call do_circles
            return

         case (IDM_ELLIPSES)
            call do_ellipses
            return

         case (IDM_POLYGONS)
            call do_polygons
            return

         case (IDM_PAINT)
            call do_paint
            return

         case (IDM_QUIT)
            iret = SendMessage(window,WM_CLOSE,0,0)
            return
      end select

   case (WM_PAINT)
      iret = BeginPaint(window,ps)
      call blit
      iret = EndPaint(window,ps)
      return

   case (WM_SETFOCUS)
      call fg_realize(hpal)
      iret = InvalidateRect(window,NULL_RECT,.TRUE.)
      return

   case (WM_SIZE)
      cxClient = LOWORD(lParam)
      cyClient = HIWORD(lParam)
      return

   case (WM_DESTROY)
      call fg_vbclose
      call fg_vbfree(hvb)
      call fg_vbfin
      iret = DeleteObject(hpal)
      iret = ReleaseDC(window,hdc)
      call PostQuitMessage(0)
      return
end select

WindowProc = DefWindowProc(window,message,wParam,lParam)
return
end

!*****************************************************************************
!                                                                            *
!  blit                                                                      *
!                                                                            *
!  Use fg_vbpaste() or fg_vbscale() to display the virtual buffer contents   *
!  in the client area, depending on the size of the client window.           *
!                                                                            *
!*****************************************************************************

subroutine blit
use fgwin

include 'graphics.fi'

if (cxClient > WIDTH .or. cyClient > HEIGHT) then ! window larger than 640x480
   call fg_vbscale(0,WIDTH-1,0,HEIGHT-1,0,cxClient-1,0,cyClient-1)
else
   call fg_vbpaste(0,WIDTH-1,0,HEIGHT-1,0,cyClient-1)
end if

return
end

!*****************************************************************************
!                                                                            *
!  do_circles                                                                *
!                                                                            *
!  Draw a series of concentric circles.                                      *
!                                                                            *
!*****************************************************************************

subroutine do_circles
use fgwin

include 'graphics.fi'

integer*4 i, radius

call fg_setcolor(11)
call fg_fillpage

! draw 25 concentric circles at the center of the virtual buffer

call fg_move(WIDTH/2,HEIGHT/2)
radius = 4
call fg_setcolor(25)
do i = 1,25
   call fg_circle(radius)
   radius = radius + 8
end do

call blit

return
end

!*****************************************************************************
!                                                                            *
!  do_ellipses                                                               *
!                                                                            *
!  Draw a series of concentric ellipses.                                     *
!                                                                            *
!*****************************************************************************

subroutine do_ellipses
use fgwin

include 'graphics.fi'

integer*4 i
integer*4 horiz, vert

call fg_setcolor(11)
call fg_fillpage

! draw 80 concentric ellipses at the center of the virtual buffer

call fg_move(WIDTH/2,HEIGHT/2)
horiz = 4
vert  = 1
call fg_setcolor(25)
do i = 1,80
   call fg_ellipse(horiz,vert)
   horiz = horiz + 3
   vert  = vert + 1
end do

call blit

return
end

!*****************************************************************************
!                                                                            *
!  do_lines                                                                  *
!                                                                            *
!  Draw a pattern of solid lines.                                            *
!                                                                            *
!*****************************************************************************

subroutine do_lines
use fgwin

include 'graphics.fi'

integer*4 x, y
integer*4 i, x1, x2, y1
integer*4 line_color(8)

data line_color /12,11,19,21,21,19,11,12/

call fg_setcolor(25)
call fg_fillpage

! draw horizontal lines

do y = 0,HEIGHT-1,40
   do i = 1,8
      call fg_setcolor(line_color(i))
      y1 = y + 3*i
      call fg_move(0,y1)
      call fg_draw(WIDTH-1,y1)
   end do
end do

! draw vertical lines

do x = 0,WIDTH-1,60
   do i = 1,8
      call fg_setcolor(line_color(i))
      x1 = x + 3*i
      call fg_move(x1,0)
      call fg_draw(x1,HEIGHT-1)
   end do
end do

! draw red diagonal lines

call fg_setcolor(22)
do x1 = -640,640-60,60
   x2 = x1 + HEIGHT
   call fg_move(x1,0)
   call fg_draw(x2,HEIGHT)
end do
do x1 = 0,1280-60,60
   x2 = x1 - HEIGHT
   call fg_move(x1,0)
   call fg_draw(x2,HEIGHT)
end do

call blit

return
end

!*****************************************************************************
!                                                                            *
!  do_paint                                                                  *
!                                                                            *
!  Demonstrate region fill.                                                  *
!                                                                            *
!*****************************************************************************

subroutine do_paint
use fgwin

include 'graphics.fi'

integer*4 x1, x2, y1, y2

call fg_setcolor(25)
call fg_fillpage

! draw a rectangle

x1 = 40
x2 = WIDTH - 40
y1 = 20
y2 = HEIGHT - 20
call fg_setcolor(21)
call fg_rect(x1,x2,y1,y2)

! outline the rectangle

call fg_setcolor(10)
call fg_box(x1,x2,y1,y2)

! draw the circle

x1 = WIDTH / 2
y1 = HEIGHT / 2
call fg_move(x1,y1)
call fg_circle(80)

! draw cross bars in the circle

call fg_move(x1,y1-80)
call fg_draw(x1,y1+80)
call fg_move(x1-80,y1)
call fg_draw(x1+80,y1)

! paint each quarter of the circle

call fg_setcolor(11)
call fg_paint(x1-6,y1-6)
call fg_setcolor(12)
call fg_paint(x1+6,y1+6)
call fg_setcolor(13)
call fg_paint(x1+6,y1-6)
call fg_setcolor(14)
call fg_paint(x1-6,y1+6)

! paint the area outside the box

call fg_setcolor(24)
call fg_paint(41,21)

call blit

return
end

!*****************************************************************************
!                                                                            *
!  do_points                                                                 *
!                                                                            *
!  Draw a pattern of points.                                                 *
!                                                                            *
!*****************************************************************************

subroutine do_points
use fgwin

include 'graphics.fi'

integer*4 x, y

! fill the virtual buffer with blue pixels

call fg_setcolor(24)
call fg_fillpage

! draw the patterns of points

call fg_setcolor(19)
do x = 7,WIDTH-20,20
   do y = 0,HEIGHT-8,8
      call fg_point(x,y)
   end do
end do

call fg_setcolor(22)
do x = 17,WIDTH-20,20
   do y = 4,HEIGHT-8,8
      call fg_point(x,y)
   end do
end do

call blit

return
end

!*****************************************************************************
!                                                                            *
!  do_polygons                                                               *
!                                                                            *
!  Draw a grid of filled polygons.                                           *
!                                                                            *
!*****************************************************************************

subroutine do_polygons
use fgwin

include 'graphics.fi'

integer*4 i, j

integer*4 xy_dkblue(8), xy_ltblue(8), xy_magenta(8), work_array(57*2)

data xy_dkblue  /0,16, 24,0, 24,40, 0,56/
data xy_ltblue  /24,0, 72,0, 72,40, 24,40/
data xy_magenta /0,56, 24,40, 72,40, 48,56/

call fg_setcolor(25)
call fg_fillpage

! draw 225 filled polygons (15 rows of 15)

do j = 0,14
   do i = 0,14
      call fg_polyoff(i*72-j*24,j*56-i*16)
      call fg_setcolor(11)
      call fg_polyfill(xy_dkblue,work_array,4)
      call fg_setcolor(19)
      call fg_polyfill(xy_ltblue,work_array,4)
      call fg_setcolor(20)
      call fg_polyfill(xy_magenta,work_array,4)
   end do
end do

call blit

return
end

!*****************************************************************************
!                                                                            *
!  do_rectangles                                                             *
!                                                                            *
!  Draw a grid of filled rectangles.                                         *
!                                                                            *
!*****************************************************************************

subroutine do_rectangles
use fgwin

include 'graphics.fi'

integer*4 i, j
integer*4 color
integer*4 x1, x2, y1, y2
integer*4 xinc, yinc

x1 = 0
xinc = WIDTH / 10
x2 = xinc - 1

y1 = 0
yinc = HEIGHT / 10
y2 = yinc - 1

color = 10

! draw 100 filled rectangles (10 rows of 10)

do i = 1,10
   do j = 1,10
      call fg_setcolor(color)
      call fg_rect(x1,x2,y1,y2)
      color = color + 1
      if (color > 24) color = 10
      x1 = x1 + xinc
      x2 = x2 + xinc
   end do
   x1 = 0
   x2 = xinc - 1
   y1 = y1 + yinc
   y2 = y2 + yinc
end do

call blit

return
end
