!*****************************************************************************
!                                                                            *
!  SCROLLER.F90                                                              *
!                                                                            *
!  This program demonstrates circular scrolling within a virtual buffer.     *
!                                                                            *
!  Because fg_scroll() requires a "hidden" virtual buffer to save and then   *
!  restore portions of the area being scrolled, we create a second virtual   *
!  buffer for this purpose.                                                  *
!                                                                            *
!*****************************************************************************

integer*4 function WinMain (hInstance, hPrevInstance, lpszCmdParam, nCmdShow)
!MS$ATTRIBUTES STDCALL, ALIAS : '_WinMain@16' :: WinMain
use msfwina
use fgwin

interface
integer*4 function WindowProc (window, message, wParam, lParam)
!MS$ATTRIBUTES STDCALL, ALIAS:'_WindowProc@16' :: WindowProc
integer*4 window, message, wParam, lParam
end function WindowProc
end interface

integer*4 hInstance, hPrevInstance, lpszCmdParam, nCmdShow

include 'scroller.fi'

character*12      AppName
integer*4         iret
integer*4         window
type (T_MSG)      msg
type (T_WNDCLASS) wndclass

data AppName /'FGscroller'C/

if (hPrevInstance == 0) then
   wndclass%style         = CS_HREDRAW .or. CS_VREDRAW
   wndclass%lpfnWndProc   = LOC(WindowProc)
   wndclass%cbClsExtra    = 0
   wndclass%cbWndExtra    = 0
   wndclass%hInstance     = hInstance
   wndclass%hIcon         = LoadIcon(NULL,IDI_APPLICATION)
   wndclass%hCursor       = LoadCursor(NULL,IDC_ARROW)
   wndclass%hbrBackground = NULL
   wndclass%lpszMenuName  = NULL
   wndclass%lpszClassName = LOC(AppName)
   iret = RegisterClass(wndclass)
   lpszCmdParam = lpszCmdParam
end if

window = CreateWindow(      &
   AppName,                 &  ! window class name
   'Display Strings in Client Area'C, &  ! window caption
   WS_OVERLAPPEDWINDOW,     &  ! window style
   CW_USEDEFAULT,           &  ! initial x position
   CW_USEDEFAULT,           &  ! initial y position
   WIDTH,                   &  ! initial x size
   HEIGHT,                  &  ! initial y size
   NULL,                    &  ! parent window handle
   NULL,                    &  ! window menu handle
   hInstance,               &  ! program instance handle
   NULL)                       ! creation parameters

iret = ShowWindow(window,nCmdShow)
iret = UpdateWindow(window)

! The message loop processes entries placed in the message queue.
! When no message is ready, call scroll() to perform one frame of
! animation.

do while (.TRUE.)
   if (PeekMessage(msg,NULL,0,0,PM_REMOVE)) then
      if (msg.message == WM_QUIT) then
         WinMain = msg%wParam
         return
      else
         iret = TranslateMessage(msg)
         iret = DispatchMessage(msg)
      end if
   else
      call scroll
   end if
end do

end

!*****************************************************************************
!                                                                            *
!  WindowProc                                                                *
!                                                                            *
!  Window procedure to handle messages sent to the window.                   *
!                                                                            *
!*****************************************************************************

integer*4 function WindowProc (window, message, wParam, lParam)
!MS$ATTRIBUTES STDCALL, ALIAS:'_WindowProc@16' :: WindowProc
use msfwina
use fgwin
integer*4 window, message, wParam, lParam

include 'scroller.fi'

type (T_PAINTSTRUCT) ps
integer*4 hdc, hpal, hvb1, hvb2
integer*4 iret

integer*4 cxClient, cyClient
common    cxClient, cyClient

save hdc, hpal, hvb1, hvb2

WindowProc = 0

select case (message)
   case (WM_CREATE)
      hdc = GetDC(window)
      call fg_setdc(hdc)
      hpal = fg_defpal()
      call fg_realize(hpal)

      call fg_vbinit
      hvb1 = fg_vballoc(WIDTH,HEIGHT)
      hvb2 = fg_vballoc(WIDTH,HEIGHT)
      iret = fg_vbopen(hvb2)
      call fg_vbcolors
      iret = fg_vbopen(hvb1)
      call fg_vbcolors
      call fg_sethpage(hvb2)

      call fg_setcolor(19)
      call fg_fillpage
      call fg_setcolor(25)
      call fg_rect(132,188,50,150)

      call fg_setcolor(20)
      call fg_move(160,67)
      call fg_draw(175,107)
      call fg_draw(140,82)
      call fg_draw(180,82)
      call fg_draw(145,107)
      call fg_draw(160,67)
      call fg_paint(160,77)
      call fg_paint(150,87)
      call fg_paint(160,87)
      call fg_paint(170,87)
      call fg_paint(155,97)
      call fg_paint(165,97)
      return

   case (WM_PAINT)
      iret = BeginPaint(window,ps)
      call fg_vbscale(0,WIDTH-1,0,HEIGHT-1,0,cxClient-1,0,cyClient-1)
      iret = EndPaint(window,ps)
      return

   case (WM_SETFOCUS)
      call fg_realize(hpal)
      iret = InvalidateRect(window,NULL_RECT,.TRUE.)
      return

   case (WM_SIZE)
      cxClient = LOWORD(lParam)
      cyClient = HIWORD(lParam)
      return

   case (WM_DESTROY)
      call fg_vbclose
      call fg_vbfree(hvb1)
      call fg_vbfree(hvb2)
      call fg_vbfin
      iret = DeleteObject(hpal)
      iret = ReleaseDC(window,hdc)
      call PostQuitMessage(0)
      return
end select

WindowProc = DefWindowProc(window,message,wParam,lParam)
return
end

!*****************************************************************************
!                                                                            *
!  The scroll() subroutine moves the scrolling region up four pixels in a    *
!  circular manner. It is called from the message loop in WinMain when no    *
!  messages are waiting.                                                     *
!                                                                            *
!*****************************************************************************

subroutine scroll
use fgwin

include 'scroller.fi'

integer*4 cxClient, cyClient
common    cxClient, cyClient

call fg_scroll(136,184,50,150,-4,0)
call fg_vbscale(0,WIDTH-1,0,HEIGHT-1,0,cxClient-1,0,cyClient-1)

return
end
